import Vue from 'vue';

var script = /*#__PURE__*/Vue.extend({
  name: 'VueChillCodeInput',
  model: {
    prop: 'valueModel',
    event: 'update:valueModel'
  },
  props: {
    length: {
      type: Number,
      required: false,
      default: 4
    },
    valueModel: {
      type: String,
      required: false,
      default: ''
    },
    autoFocus: {
      type: Boolean,
      required: false,
      default: false
    },
    autoBlur: {
      type: Boolean,
      required: false,
      default: true
    },
    type: {
      type: String,
      required: false,
      default: 'text',
      validator: value => ['text', 'number', 'tel'].includes(value)
    },
    password: {
      type: Boolean,
      required: false,
      default: false
    }
  },
  data: function () {
    return {
      activeColumnIndex: null,
      content: '',
      dontEmit: false
    };
  },
  computed: {
    currentColumn: function () {
      if (this.activeColumnIndex === null) {
        return null;
      }

      return this.$refs[`field__${this.activeColumnIndex}`][0];
    },
    getType: function () {
      return this.password ? 'password' : this.type;
    }
  },
  methods: {
    blur: function () {
      var _this$currentColumn;

      (_this$currentColumn = this.currentColumn) === null || _this$currentColumn === void 0 ? void 0 : _this$currentColumn.blur();
    },
    focus: function () {
      this.setActiveIndex(0);
      this.currentColumn.focus();
    },
    clear: function () {
      for (let i = 0; i < this.length; i++) {
        this.getColumn(i).value = '';
      }

      this.computeContent();
    },
    setActiveIndex: function (index) {
      this.activeColumnIndex = index;
    },
    getColumn: function (index) {
      var _this$$refs$;

      return ((_this$$refs$ = this.$refs[`field__${index}`]) === null || _this$$refs$ === void 0 ? void 0 : _this$$refs$[0]) || null;
    },
    onBackspace: function (event, index) {
      this.setActiveIndex(index);
      this.$refs[`field__${index}`][0].value = '';
      event.preventDefault();

      if (index > 0) {
        const input = this.getColumn(index - 1);
        this.dontEmit = true;
        input === null || input === void 0 ? void 0 : input.focus();
        this.dontEmit = false;
      }
    },
    onDelete: function (index) {
      this.setActiveIndex(index);
    },
    onInput: function (index) {
      this.dontEmit = true;
      this.setActiveIndex(index);
      const column = this.getColumn(index);
      const value = column.value;

      if (value.length > 1) {
        column.value = value[1];
      }

      this.computeContent();

      if (value.trim()) {
        const nextInput = this.getColumn(index + 1);

        if (nextInput) {
          nextInput.focus();
        } else {
          if (this.autoBlur) {
            this.$refs[`field__${index}`][0].blur();
          }

          this.$emit('done', this.content);
        }
      }

      this.dontEmit = false;
    },
    onBlur: function () {
      if (this.dontEmit) {
        return;
      }

      this.setActiveIndex(null);
      this.$emit('blur');
    },
    computeContent: function () {
      this.content = Object.values(this.$refs).map(i => i[0].value).join('').trim();
      this.$emit('update:valueModel', this.content);
    },
    onPaste: function (event) {
      // @ts-ignore
      const content = (event.clipboardData || window.clipboardData).getData('text').trim();
      this.fillFields(content);
    },
    fillFields: function (content) {
      if (!(content !== null && content !== void 0 && content.length)) {
        return;
      }

      this.dontEmit = true;

      for (let i = 0; i < this.length; i++) {
        this.getColumn(i).value = content[i];
        this.computeContent();
        const nextIndex = i + 1;
        if (nextIndex) if (nextIndex === this.length) {
          if (this.autoBlur) {
            this.getColumn(i).blur();
            this.$emit('done', this.content);
          }

          break;
        } else {
          var _this$getColumn;

          (_this$getColumn = this.getColumn(nextIndex)) === null || _this$getColumn === void 0 ? void 0 : _this$getColumn.focus();
        }
      }

      this.dontEmit = false;
    }
  },
  mounted: function () {
    this.fillFields(this.value);

    if (this.autoFocus) {
      this.focus();
    }
  }
});

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

const isOldIE = typeof navigator !== 'undefined' &&
    /msie [6-9]\\b/.test(navigator.userAgent.toLowerCase());
function createInjector(context) {
    return (id, style) => addStyle(id, style);
}
let HEAD;
const styles = {};
function addStyle(id, css) {
    const group = isOldIE ? css.media || 'default' : id;
    const style = styles[group] || (styles[group] = { ids: new Set(), styles: [] });
    if (!style.ids.has(id)) {
        style.ids.add(id);
        let code = css.source;
        if (css.map) {
            // https://developer.chrome.com/devtools/docs/javascript-debugging
            // this makes source maps inside style tags work properly in Chrome
            code += '\n/*# sourceURL=' + css.map.sources[0] + ' */';
            // http://stackoverflow.com/a/26603875
            code +=
                '\n/*# sourceMappingURL=data:application/json;base64,' +
                    btoa(unescape(encodeURIComponent(JSON.stringify(css.map)))) +
                    ' */';
        }
        if (!style.element) {
            style.element = document.createElement('style');
            style.element.type = 'text/css';
            if (css.media)
                style.element.setAttribute('media', css.media);
            if (HEAD === undefined) {
                HEAD = document.head || document.getElementsByTagName('head')[0];
            }
            HEAD.appendChild(style.element);
        }
        if ('styleSheet' in style.element) {
            style.styles.push(code);
            style.element.styleSheet.cssText = style.styles
                .filter(Boolean)
                .join('\n');
        }
        else {
            const index = style.ids.size - 1;
            const textNode = document.createTextNode(code);
            const nodes = style.element.childNodes;
            if (nodes[index])
                style.element.removeChild(nodes[index]);
            if (nodes.length)
                style.element.insertBefore(textNode, nodes[index]);
            else
                style.element.appendChild(textNode);
        }
    }
}

/* script */
const __vue_script__ = script;
/* template */

var __vue_render__ = function () {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    staticClass: "holder"
  }, _vm._l(Array.from(Array(_vm.length).keys()), function (index) {
    return _c('input', {
      key: index,
      ref: "field__" + index,
      refInFor: true,
      attrs: {
        "type": _vm.getType
      },
      on: {
        "input": function ($event) {
          $event.stopPropagation();
          return _vm.onInput(index);
        },
        "keydown": [function ($event) {
          if (!$event.type.indexOf('key') && _vm._k($event.keyCode, "backspace", undefined, $event.key, undefined)) {
            return null;
          }

          return _vm.onBackspace($event, index);
        }, function ($event) {
          if (!$event.type.indexOf('key') && _vm._k($event.keyCode, "delete", [8, 46], $event.key, ["Backspace", "Delete", "Del"])) {
            return null;
          }

          return _vm.onDelete(index);
        }],
        "paste": function ($event) {
          $event.preventDefault();
          return _vm.onPaste($event);
        },
        "blur": function ($event) {
          $event.preventDefault();
          return _vm.onBlur($event);
        },
        "focus": function ($event) {
          $event.preventDefault();
          return _vm.setActiveIndex(index);
        }
      }
    });
  }), 0);
};

var __vue_staticRenderFns__ = [];
/* style */

const __vue_inject_styles__ = function (inject) {
  if (!inject) return;
  inject("data-v-4a8d234f_0", {
    source: ".holder[data-v-4a8d234f]{display:inline-flex;justify-content:center;padding:1em;background:#d3d3d3;border-radius:4px}input[data-v-4a8d234f]{font-size:20px;width:50px;height:50px;display:flex;justify-content:center;align-items:center;margin-right:10px;text-align:center;border-radius:6px;border:1px solid transparent;outline:transparent}input[data-v-4a8d234f]:active,input[data-v-4a8d234f]:focus{border:1px solid green}",
    map: undefined,
    media: undefined
  });
};
/* scoped */


const __vue_scope_id__ = "data-v-4a8d234f";
/* module identifier */

const __vue_module_identifier__ = undefined;
/* functional template */

const __vue_is_functional_template__ = false;
/* style inject SSR */

/* style inject shadow dom */

const __vue_component__ = /*#__PURE__*/normalizeComponent({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, createInjector, undefined, undefined);

// Import vue component

// Default export is installable instance of component.
// IIFE injects install function into component, allowing component
// to be registered via Vue.use() as well as Vue.component(),
var entry_esm = /*#__PURE__*/(() => {
  // Assign InstallableComponent type
  const installable = __vue_component__; // Attach install function executed by Vue.use()

  installable.install = Vue => {
    Vue.component('VueChillCodeInput', installable);
  };

  return installable;
})(); // It's possible to expose named exports when writing components that can
// also be used as directives, etc. - eg. import { RollupDemoDirective } from 'rollup-demo';
// export const RollupDemoDirective = directive;

export default entry_esm;
